@@ -26,7 +26,7 @@
 
 #include "haiku_usb.h"
 
-static int _errno_to_libusb(int status)
+int _errno_to_libusb(int status)
 {
 	return status;
 }
@@ -127,7 +127,7 @@ USBTransfer::Do(int fRawFD)
 			int i;
 			usb_iso_packet_descriptor *packetDescriptors = new usb_iso_packet_descriptor[fLibusbTransfer->num_iso_packets];
 			for (i = 0; i < fLibusbTransfer->num_iso_packets; i++) {
-				if ((fLibusbTransfer->iso_packet_desc[i]).length > (unsigned int)INT16_MAX) {
+				if ((int16)(fLibusbTransfer->iso_packet_desc[i]).length != (fLibusbTransfer->iso_packet_desc[i]).length) {
 					fUsbiTransfer->transferred = -1;
 					usbi_err(TRANSFER_CTX(fLibusbTransfer), "failed isochronous transfer");
 					break;
@@ -201,7 +201,7 @@ status_t
 USBDeviceHandle::SubmitTransfer(struct usbi_transfer *itransfer)
 {
 	USBTransfer *transfer = new USBTransfer(itransfer, fUSBDevice);
-	*((USBTransfer **)usbi_get_transfer_priv(itransfer)) = transfer;
+	*((USBTransfer **)usbi_transfer_get_os_priv(itransfer)) = transfer;
 	BAutolock locker(fTransfersLock);
 	fTransfers.AddItem(transfer);
 	release_sem(fTransfersSem);
@@ -215,16 +215,16 @@ USBDeviceHandle::CancelTransfer(USBTransfer *transfer)
 	fTransfersLock.Lock();
 	bool removed = fTransfers.RemoveItem(transfer);
 	fTransfersLock.Unlock();
-	if (removed)
+	if(removed)
 		usbi_signal_transfer_completion(transfer->UsbiTransfer());
 	return LIBUSB_SUCCESS;
 }
 
 USBDeviceHandle::USBDeviceHandle(USBDevice *dev)
 	:
+	fTransfersThread(-1),
 	fUSBDevice(dev),
 	fClaimedInterfaces(0),
-	fTransfersThread(-1),
 	fInitCheck(false)
 {
 	fRawFD = open(dev->Location(), O_RDWR | O_CLOEXEC);
@@ -242,8 +242,8 @@ USBDeviceHandle::~USBDeviceHandle()
 {
 	if (fRawFD > 0)
 		close(fRawFD);
-	for (int i = 0; i < 32; i++) {
-		if (fClaimedInterfaces & (1U << i))
+	for(int i = 0; i < 32; i++) {
+		if (fClaimedInterfaces & (1 << i))
 			ReleaseInterface(i);
 	}
 	delete_sem(fTransfersSem);
@@ -252,27 +252,27 @@ USBDeviceHandle::~USBDeviceHandle()
 }
 
 int
-USBDeviceHandle::ClaimInterface(uint8 inumber)
+USBDeviceHandle::ClaimInterface(int inumber)
 {
 	int status = fUSBDevice->ClaimInterface(inumber);
 	if (status == LIBUSB_SUCCESS)
-		fClaimedInterfaces |= (1U << inumber);
+		fClaimedInterfaces |= (1 << inumber);
 	return status;
 }
 
 int
-USBDeviceHandle::ReleaseInterface(uint8 inumber)
+USBDeviceHandle::ReleaseInterface(int inumber)
 {
 	fUSBDevice->ReleaseInterface(inumber);
-	fClaimedInterfaces &= ~(1U << inumber);
+	fClaimedInterfaces &= ~(1 << inumber);
 	return LIBUSB_SUCCESS;
 }
 
 int
-USBDeviceHandle::SetConfiguration(uint8 config)
+USBDeviceHandle::SetConfiguration(int config)
 {
 	int config_index = fUSBDevice->CheckInterfacesFree(config);
-	if (config_index == LIBUSB_ERROR_BUSY || config_index == LIBUSB_ERROR_NOT_FOUND)
+	if(config_index == LIBUSB_ERROR_BUSY || config_index == LIBUSB_ERROR_NOT_FOUND)
 		return config_index;
 	usb_raw_command command;
 	command.config.config_index = config_index;
@@ -280,12 +280,12 @@ USBDeviceHandle::SetConfiguration(uint8 config)
 			command.config.status != B_USB_RAW_STATUS_SUCCESS) {
 		return _errno_to_libusb(command.config.status);
 	}
-	fUSBDevice->SetActiveConfiguration((uint8)config_index);
+	fUSBDevice->SetActiveConfiguration(config_index);
 	return LIBUSB_SUCCESS;
 }
 
 int
-USBDeviceHandle::SetAltSetting(uint8 inumber, uint8 alt)
+USBDeviceHandle::SetAltSetting(int inumber, int alt)
 {
 	usb_raw_command command;
 	command.alternate.config_index = fUSBDevice->ActiveConfigurationIndex();
@@ -295,44 +295,27 @@ USBDeviceHandle::SetAltSetting(uint8 inumber, uint8 alt)
 		usbi_err(NULL, "Error retrieving active alternate interface");
 		return _errno_to_libusb(command.alternate.status);
 	}
-	if (command.alternate.alternate_info == (uint32)alt) {
-		usbi_dbg(NULL, "Setting alternate interface successful");
+	if (command.alternate.alternate_info == alt) {
+		usbi_dbg("Setting alternate interface successful");
 		return LIBUSB_SUCCESS;
 	}
 	command.alternate.alternate_info = alt;
 	if (ioctl(fRawFD, B_USB_RAW_COMMAND_SET_ALT_INTERFACE, &command, sizeof(command)) ||
-			command.alternate.status != B_USB_RAW_STATUS_SUCCESS) { //IF IOCTL FAILS DEVICE DISCONNECTED PROBABLY
+			command.alternate.status != B_USB_RAW_STATUS_SUCCESS) { //IF IOCTL FAILS DEVICE DISONNECTED PROBABLY
 		usbi_err(NULL, "Error setting alternate interface");
 		return _errno_to_libusb(command.alternate.status);
 	}
-	usbi_dbg(NULL, "Setting alternate interface successful");
-	return LIBUSB_SUCCESS;
-}
-
-int
-USBDeviceHandle::ClearHalt(uint8 endpoint)
-{
-	usb_raw_command command;
-	command.control.request_type = USB_REQTYPE_ENDPOINT_OUT;
-	command.control.request = USB_REQUEST_CLEAR_FEATURE;
-	command.control.value = USB_FEATURE_ENDPOINT_HALT;
-	command.control.index = endpoint;
-	command.control.length = 0;
-
-	if (ioctl(fRawFD, B_USB_RAW_COMMAND_CONTROL_TRANSFER, &command, sizeof(command)) ||
-			command.control.status != B_USB_RAW_STATUS_SUCCESS) {
-		return _errno_to_libusb(command.control.status);
-	}
+	usbi_dbg("Setting alternate interface successful");
 	return LIBUSB_SUCCESS;
 }
 
 
 USBDevice::USBDevice(const char *path)
 	:
-	fClaimedInterfaces(0),
-	fConfigurationDescriptors(NULL),
-	fActiveConfiguration(0),	//0?
 	fPath(NULL),
+	fActiveConfiguration(0),	//0?
+	fConfigurationDescriptors(NULL),
+	fClaimedInterfaces(0),
 	fEndpointToIndex(NULL),
 	fEndpointToInterface(NULL),
 	fInitCheck(false)
@@ -345,7 +328,7 @@ USBDevice::~USBDevice()
 {
 	free(fPath);
 	if (fConfigurationDescriptors) {
-		for (uint8 i = 0; i < fDeviceDescriptor.num_configurations; i++) {
+		for(int i = 0; i < fDeviceDescriptor.num_configurations; i++) {
 			if (fConfigurationDescriptors[i])
 				delete fConfigurationDescriptors[i];
 		}
@@ -382,7 +365,7 @@ USBDevice::Descriptor() const
 }
 
 const usb_configuration_descriptor *
-USBDevice::ConfigurationDescriptor(uint8 index) const
+USBDevice::ConfigurationDescriptor(uint32 index) const
 {
 	if (index > CountConfigurations())
 		return NULL;
@@ -395,42 +378,43 @@ USBDevice::ActiveConfiguration() const
 	return (usb_configuration_descriptor *) fConfigurationDescriptors[fActiveConfiguration];
 }
 
-uint8
+int
 USBDevice::ActiveConfigurationIndex() const
 {
 	return fActiveConfiguration;
 }
 
-int USBDevice::ClaimInterface(uint8 interface)
+int USBDevice::ClaimInterface(int interface)
 {
 	if (interface > ActiveConfiguration()->number_interfaces)
 		return LIBUSB_ERROR_NOT_FOUND;
-	if (fClaimedInterfaces & (1U << interface))
+	if (fClaimedInterfaces & (1 << interface))
 		return LIBUSB_ERROR_BUSY;
-	fClaimedInterfaces |= (1U << interface);
+	fClaimedInterfaces |= (1 << interface);
 	return LIBUSB_SUCCESS;
 }
 
-int USBDevice::ReleaseInterface(uint8 interface)
+int USBDevice::ReleaseInterface(int interface)
 {
-	fClaimedInterfaces &= ~(1U << interface);
+	fClaimedInterfaces &= ~(1 << interface);
 	return LIBUSB_SUCCESS;
 }
 
 int
-USBDevice::CheckInterfacesFree(uint8 config)
+USBDevice::CheckInterfacesFree(int config)
 {
 	if (fConfigToIndex.count(config) == 0)
 		return LIBUSB_ERROR_NOT_FOUND;
 	if (fClaimedInterfaces == 0)
-		return fConfigToIndex[config];
+		return fConfigToIndex[(uint8)config];
 	return LIBUSB_ERROR_BUSY;
 }
 
-void
-USBDevice::SetActiveConfiguration(uint8 config_index)
+int
+USBDevice::SetActiveConfiguration(int config_index)
 {
 	fActiveConfiguration = config_index;
+	return LIBUSB_SUCCESS;
 }
 
 uint8
@@ -462,7 +446,7 @@ USBDevice::Initialise()		//Do we need more error checking, etc? How to report?
 	fConfigurationDescriptors = new(std::nothrow) unsigned char *[fDeviceDescriptor.num_configurations];
 	fEndpointToIndex = new(std::nothrow) map<uint8,uint8> [fDeviceDescriptor.num_configurations];
 	fEndpointToInterface = new(std::nothrow) map<uint8,uint8> [fDeviceDescriptor.num_configurations];
-	for (uint8 i = 0; i < fDeviceDescriptor.num_configurations; i++) {
+	for (int i = 0; i < fDeviceDescriptor.num_configurations; i++) {
 		usb_configuration_descriptor tmp_config;
 		command.config.descriptor = &tmp_config;
 		command.config.config_index = i;
@@ -474,18 +458,19 @@ USBDevice::Initialise()		//Do we need more error checking, etc? How to report?
 		}
 		fConfigToIndex[tmp_config.configuration_value] = i;
 		fConfigurationDescriptors[i] = new(std::nothrow) unsigned char[tmp_config.total_length];
-
-		command.config_etc.descriptor = (usb_configuration_descriptor*)fConfigurationDescriptors[i];
-		command.config_etc.length = tmp_config.total_length;
-		command.config_etc.config_index = i;
-		if (ioctl(fRawFD, B_USB_RAW_COMMAND_GET_CONFIGURATION_DESCRIPTOR_ETC, &command, sizeof(command)) ||
-				command.config_etc.status != B_USB_RAW_STATUS_SUCCESS) {
+		command.control.request_type = 128;
+		command.control.request = 6;
+		command.control.value = (2 << 8) | i;
+		command.control.index = 0;
+		command.control.length = tmp_config.total_length;
+		command.control.data = fConfigurationDescriptors[i];
+		if (ioctl(fRawFD, B_USB_RAW_COMMAND_CONTROL_TRANSFER, &command, sizeof(command)) ||
+				command.control.status!=B_USB_RAW_STATUS_SUCCESS) {
 			usbi_err(NULL, "failed retrieving full configuration descriptor");
 			close(fRawFD);
 			return B_ERROR;
 		}
-
-		for (uint8 j = 0; j < tmp_config.number_interfaces; j++) {
+		for (int j = 0; j < tmp_config.number_interfaces; j++) {
 			command.alternate.config_index = i;
 			command.alternate.interface_index = j;
 			if (ioctl(fRawFD, B_USB_RAW_COMMAND_GET_ALT_INTERFACE_COUNT, &command, sizeof(command)) ||
@@ -494,8 +479,8 @@ USBDevice::Initialise()		//Do we need more error checking, etc? How to report?
 				close(fRawFD);
 				return B_ERROR;
 			}
-			uint8 num_alternate = (uint8)command.alternate.alternate_info;
-			for (uint8 k = 0; k < num_alternate; k++) {
+			int num_alternate = command.alternate.alternate_info;
+			for (int k = 0; k < num_alternate; k++) {
 				usb_interface_descriptor tmp_interface;
 				command.interface_etc.config_index = i;
 				command.interface_etc.interface_index = j;
@@ -507,7 +492,7 @@ USBDevice::Initialise()		//Do we need more error checking, etc? How to report?
 					close(fRawFD);
 					return B_ERROR;
 				}
-				for (uint8 l = 0; l < tmp_interface.num_endpoints; l++) {
+				for (int l = 0; l < tmp_interface.num_endpoints; l++) {
 					usb_endpoint_descriptor tmp_endpoint;
 					command.endpoint_etc.config_index = i;
 					command.endpoint_etc.interface_index = j;
